@@ -1,6 +1,5 @@
 import matplotlib.pyplot as plt
 from mpl_toolkits.mplot3d import Axes3D
-
 import re
 from struct import unpack,pack
 import time
@@ -14,6 +13,9 @@ from snap7.common import check_error, load_library, ipv4
 
 import threading
 import numpy as np
+from functools import partial
+
+from speedbotlib.common.cache import dson
 
 # import logging
 # logging.basicConfig(level = logging.INFO)
@@ -24,12 +26,15 @@ def rotate(origin_pos,pos,degree):
     a = pi / 180 * degree
     x2 = (pos[0] - origin_pos[0]) * cos(a) - (pos[1] - origin_pos[1]) * sin(a) + origin_pos[0]
     y2 = (pos[1] - origin_pos[1]) * cos(a) + (pos[0] - origin_pos[0]) * sin(a) + origin_pos[1]
-    return x2,y2
-
+    z2 = pos[2]
+    return x2,y2,z2
 
 class Truss():
-    def __init__(self,name,port,area_x,area_y,area_z):
+    def __init__(self,name,db_dir,port,area_x,area_y,area_z):
         self.area_x,self.area_y,self.area_z = area_x,area_y,area_z
+        self.plate_dir = f'{db_dir}/plates_sorting'
+        self.status_file = f'{db_dir}/status/{name}.json'
+        self.params_file = f'{db_dir}/params/{name}.json'
 
         self.server = snap7.server.Server()
         self.dbs = dict()
@@ -55,7 +60,7 @@ class Truss():
         self.db0[1] = 88 # B
         self.db1[34] = 0b00000010 # fault,writeable,paused,safe_a,safe_b,finished_a,finished_b,synced
         
-        self.x = [self.area_x[0],self.area_x[1]]
+        self.x = [-3400,3200]
         self.y = [self.area_y[0],self.area_y[0]]
         self.z = [self.area_z[1],self.area_z[1]]
         self.r = [0,0]
@@ -169,7 +174,7 @@ class Truss():
                 self.y[index] = interval
                 await asyncio.sleep(0.01)
         async def z(self,start,end):
-            for interval in np.linspace(start,end,int(abs(start - end) / 10)):
+            for interval in np.linspace(start,end,int(abs(start - end) / 40)):
                 if index == 0: self.db1[10],self.db1[11],self.db1[12],self.db1[13] = pack('!f',interval)
                 elif index == 1: self.db1[26],self.db1[27],self.db1[28],self.db1[29] = pack('!f',interval)
                 self.z[index] = interval
@@ -190,22 +195,18 @@ class Truss():
             synced[index] = 2
             while synced[index] > synced[index2]: await asyncio.sleep(0.1)
         await self.loop.create_task(z(self,z_end,self.area_z[1]))
-
         if synced: 
             synced[index]= 3
             while synced[index] > synced[index2]: await asyncio.sleep(0.1)
         await self.loop.create_task(asyncio.wait([x(self,x_end,x2_end),y(self,y_end,y2_end),r(self,r_end,r2_end)]))
-
         if synced: 
             synced[index] = 4
             while synced[index] > synced[index2]: await asyncio.sleep(0.1)
         await self.loop.create_task(z(self,self.area_z[1],z2_end))
-
         if synced: 
             synced[index] = 5
             while synced[index] > synced[index2]: await asyncio.sleep(0.1)
         await self.loop.create_task(z(self,z2_end,self.area_z[1]))
-
         if synced: 
             synced[index] = 6
             while synced[index] > synced[index2]: await asyncio.sleep(0.1)
@@ -221,7 +222,7 @@ class Truss():
             if not event: break
 
             request = self.server.event_text(event)
-            print(request)
+            
             m = re.match(r'[0-9]{4}-[0-9]{2}-[0-9]{2}\s[0-9]{2}:[0-9]{2}:[0-9]{2} \[[\d]+\.[\d]+\.[\d]+.[\d]+\] (\w+) request, Area : DB(\d+), Start : (\d+), Size : (\d+) --> OK',request)
             if not m: continue
             command = m.group(1)
@@ -230,9 +231,8 @@ class Truss():
             size = int(m.group(4))
             if command != 'Write': continue
             if index > 1: continue
+            print(request)
             action = self.db0[index]
-
-            from functools import partial
             def finish(db,index,f):
                 db[index] = 88
 
@@ -262,7 +262,7 @@ class Truss():
             elif action == 3:
                 print('复位指令')
                 if index == 0:
-                    self.db0[38],self.db0[39],self.db0[40],self.db0[41] = pack('!f',3400)
+                    self.db0[38],self.db0[39],self.db0[40],self.db0[41] = pack('!f',-3400)
                     self.db0[42],self.db0[43],self.db0[44],self.db0[45] = pack('!f',self.area_y[0])
                     self.db0[46],self.db0[47],self.db0[48],self.db0[49] = pack('!f',self.area_z[1])
                     self.db0[50],self.db0[51],self.db0[52],self.db0[53] = pack('!f',0)
@@ -280,6 +280,12 @@ class Truss():
                             self.loop.create_task(self.moving_r(index))
                         ])).add_done_callback(partial(finish,self.db0,index))
                 pass
+
+            self.status = dson(self.status_file)
+            self.params = dson(self.params_file)
+            plate_name = self.status['plate_name']
+            self.plate = dson(f'{self.plate_dir}/{plate_name}.json')
+
         self.view.cla()
 
         #区域
@@ -290,22 +296,170 @@ class Truss():
         self.view.plot([0,0],self.area_y,[0,0])
         self.view.plot([0,0],[0,0],[self.area_z[0],self.area_z[1]])
 
+        try:
+            #画钢板
+            plate_corner,plate_degree = self.status['corner'],self.status['degree']
+            
+            plate_size = self.plate['size']
+            self.plate_right,self.plate_top = plate_corner
+            self.plate_left = self.plate_right - plate_size[0]
+            self.plate_bottom = self.plate_top - plate_size[1]
+            self.grab_z = self.params['sorting.grab_z'] + plate_size[2]
+            
+            plate_p0 = rotate(plate_corner,(self.plate_left,self.plate_top,self.grab_z),plate_degree)
+            plate_p1 = [plate_corner[0],plate_corner[1],self.grab_z]
+            plate_p2 = rotate(plate_corner,(self.plate_right,self.plate_bottom,self.grab_z),plate_degree)
+            plate_p3 = rotate(plate_corner,(self.plate_left,self.plate_bottom,self.grab_z),plate_degree)
+            
+            self.rectangle(plate_p0,plate_p1,plate_p2,plate_p3)
+            self.point(plate_p1,f'{plate_p1} {plate_degree}°')
+
+            #画托盘
+            drop_pos = self.params['sorting.pallet_pos']
+            pallet_size = self.params['sorting.pallet_size']
+            
+            p0 = drop_pos
+            p1 = drop_pos[0], drop_pos[1] + pallet_size[1], drop_pos[2]
+            p2 = drop_pos[0] + pallet_size[0], drop_pos[1] + pallet_size[1], drop_pos[2]
+            p3 = drop_pos[0] + pallet_size[0], drop_pos[1], drop_pos[2]
+            self.rectangle(p0,p1,p2,p3)
+            self.text(p2,str(drop_pos))
+
+            if self.plate['finished']:
+                drop_positions = self.plate['tiling'][-1]
+            else:
+                tiling_index = self.plate['tiling_index']
+                drop_positions = self.plate['tiling'][tiling_index]
+
+            for part in self.plate['parts']:
+                id = part['id']
+                handle = part['handle']
+                center = part['center']
+                part_size = part['size']
+                part_degree = part['degree']
+
+                if 'a_grab_pos' in part: a_grab_pos = part['a_grab_pos']
+                if 'a_grab_degree' in part: a_grab_degree = part['a_grab_degree']
+                if 'a_drop_pos_offset' in part: a_drop_pos_offset = part['a_drop_pos_offset']
+                if 'a_drop_degree' in part: a_drop_degree = part['a_drop_degree']
+                if 'a_magnetic_dots' in part: a_magnetic_dots = part['a_magnetic_dots']
+        
+                if 'b_grab_pos' in part: b_grab_pos = part['b_grab_pos']
+                if 'b_grab_degree' in part: b_grab_degree = part['b_grab_degree']
+                if 'b_drop_pos_offset' in part: b_drop_pos_offset = part['b_drop_pos_offset']
+                if 'b_drop_degree' in part: b_drop_degree = part['b_drop_degree']
+                if 'b_magnetic_dots' in part: b_magnetic_dots = part['b_magnetic_dots']
+
+                box_center = self.plate_left + center[0],self.plate_bottom + center[1]
+                box_left = box_center[0] - part_size[0] / 2
+                box_top = box_center[1] - part_size[1] / 2
+                box_right = box_left + part_size[0]
+                box_bottom = box_top + part_size[1]
+                
+                # 叠加偏移，以右下角作为原点
+                p0 = rotate(box_center,(box_left,box_top,self.grab_z),-part_degree)
+                p1 = rotate(box_center,(box_right,box_top,self.grab_z),-part_degree)
+                p2 = rotate(box_center,(box_right,box_bottom,self.grab_z),-part_degree)
+                p3 = rotate(box_center,(box_left,box_bottom,self.grab_z),-part_degree)
+                p0 = rotate(plate_corner,p0,plate_degree)
+                p1 = rotate(plate_corner,p1,plate_degree)
+                p2 = rotate(plate_corner,p2,plate_degree)
+                p3 = rotate(plate_corner,p3,plate_degree)
+
+                #可视化：包围盒
+                self.rectangle(p0, p1, p2, p3)
+
+                #叠加钢板角度
+                if 'a_grab_degree' in part: a_grab_degree -= plate_degree
+                if 'b_grab_degree' in part: b_grab_degree -= plate_degree
+
+                if 'a_grab_pos' in part:
+                    a_grab_pos[0] += self.plate_left
+                    a_grab_pos[1] += self.plate_bottom
+                    a_grab_pos[2] = self.grab_z
+
+                if 'b_grab_pos' in part:
+                    b_grab_pos[0] += self.plate_left
+                    b_grab_pos[1] += self.plate_bottom
+                    b_grab_pos[2] = self.grab_z
+
+                if id not in drop_positions:continue
+                if not part['picked']: continue
+
+                drop_pos,pallet_name = drop_positions[id]
+                drop_pos_offset = self.params['sorting.pallet_pos']
+                drop_pos = drop_pos_offset[0] + drop_pos[0],drop_pos_offset[1] + drop_pos[1],drop_pos_offset[2] + drop_pos[2]
+
+                if 0 == handle:
+                    a_grab_pos = rotate(plate_corner,a_grab_pos,plate_degree)
+                    
+                    drop_x = drop_pos[0] + a_drop_pos_offset[0]
+                    drop_y = drop_pos[1] + a_drop_pos_offset[1]
+                    drop_z = drop_pos[2] + part_size[2]
+                    
+                    box_left = drop_pos[0] - part_size[0] / 2  
+                    box_top = drop_pos[1] - part_size[1] / 2
+                    box_right = box_left + part_size[0]
+                    box_bottom = box_top + part_size[1]
+                    p0 = box_left,box_top,drop_z
+                    p1 = box_right,box_top,drop_z
+                    p2 = box_right,box_bottom,drop_z
+                    p3 = box_left,box_bottom,drop_z
+                    self.rectangle(p0, p1, p2, p3)
+                elif 1 == handle:
+                    b_grab_pos = rotate(plate_corner,b_grab_pos,plate_degree)
+                    drop_x = drop_pos[0] + b_drop_pos_offset[0]
+                    drop_y = drop_pos[1] + b_drop_pos_offset[1]
+                    drop_z = drop_pos[2] + part_size[2]
+
+                    box_left = drop_pos[0] - part_size[0] / 2  
+                    box_top = drop_pos[1] - part_size[1] / 2
+                    box_right = box_left + part_size[0]
+                    box_bottom = box_top + part_size[1]
+                    p0 = box_left,box_top,drop_z
+                    p1 = box_right,box_top,drop_z
+                    p2 = box_right,box_bottom,drop_z
+                    p3 = box_left,box_bottom,drop_z
+                    self.rectangle(p0, p1, p2, p3)
+                else:
+                    a_grab_pos = rotate(plate_corner,a_grab_pos,plate_degree)
+                    a_drop_x = drop_pos[0] + a_drop_pos_offset[0]
+                    a_drop_y = drop_pos[1] + a_drop_pos_offset[1]
+                    a_drop_z = drop_pos[2]
+                    b_grab_pos = rotate(plate_corner,b_grab_pos,plate_degree)
+                    b_drop_x = drop_pos[0] + b_drop_pos_offset[0]
+                    b_drop_y = drop_pos[1] + b_drop_pos_offset[1]
+                    b_drop_z = drop_pos[2]
+
+                    box_left = drop_pos[0] - part_size[0] / 2  
+                    box_top = drop_pos[1] - part_size[1] / 2
+                    box_right = box_left + part_size[0]
+                    box_bottom = box_top + part_size[1]
+                    p0 = [box_left,box_top,drop_pos[2]] 
+                    p1 = [box_right,box_top,drop_pos[2]] 
+                    p2 = [box_right,box_bottom,drop_pos[2]] 
+                    p3 = [box_left,box_bottom,drop_pos[2]]
+                    
+                    self.rectangle(p0, p1, p2, p3)
+        except:pass
+                    
         #桁架
         self.view.plot([self.x[0],self.x[0]],[self.area_y[0],self.area_y[1]],[self.area_z[1],self.area_z[1]],c='red')
         self.view.plot([self.x[1],self.x[1]],[self.area_y[0],self.area_y[1]],[self.area_z[1],self.area_z[1]],c='red')
-
-        #抓手
         self.view.plot([self.x[0],self.x[0]],[self.y[0],self.y[0]],[self.z[0],self.area_z[1]],c='red')
         self.view.plot([self.x[1],self.x[1]],[self.y[1],self.y[1]],[self.z[1],self.area_z[1]],c='red')
         
-        x0,y0=rotate([self.x[0],self.y[0]],[self.x[0]-750,self.y[0]],-self.r[0])
-        x1,y1=rotate([self.x[0],self.y[0]],[self.x[0]+750,self.y[0]],-self.r[0])
+        #抓手A
+        x0,y0,_=rotate([self.x[0],self.y[0],0],[self.x[0]-750,self.y[0],0],-self.r[0])
+        x1,y1,_=rotate([self.x[0],self.y[0],0],[self.x[0]+750,self.y[0],0],-self.r[0])
         self.view.plot([x0,x1],[y0,y1],[self.z[0],self.z[0]],c='blue')
 
-        x0,y0=rotate([self.x[1],self.y[1]],[self.x[1]-750,self.y[1]],-self.r[1])
-        x1,y1=rotate([self.x[1],self.y[1]],[self.x[1]+750,self.y[1]],-self.r[1])
+        #抓手B
+        x0,y0,_=rotate([self.x[1],self.y[1],0],[self.x[1]-750,self.y[1],0],-self.r[1])
+        x1,y1,_=rotate([self.x[1],self.y[1],0],[self.x[1]+750,self.y[1],0],-self.r[1])
         self.view.plot([x0,x1],[y0,y1],[self.z[1],self.z[1]],c='blue')
 
+        #位置
         self.view.text(self.x[0],self.y[0],self.z[0],str([int(self.x[0]),int(self.y[0]),int(self.z[0]),int(self.r[0])]))
         self.view.text(self.x[1],self.y[1],self.z[1],str([int(self.x[1]),int(self.y[1]),int(self.z[1]),int(self.r[1])]))
 
@@ -314,6 +468,19 @@ class Truss():
         await asyncio.sleep(0.001)
         
         self.loop.create_task(self.update())
+    def text(self,p,s=''):
+        self.view.text(p[0],p[1],p[2],s=s)
+    def point(self,p,s=''):
+        if p:
+            self.view.text(p[0],p[1],p[2],s=s)
+            self.view.scatter(p[0],p[1],p[2])
+    def rectangle(self,p0,p1,p2,p3,c=None,s=''):
+        #可视化：钢板
+        self.view.plot(
+            [p0[0],p1[0],p2[0],p3[0],p0[0]],
+            [p0[1],p1[1],p2[1],p3[1],p0[1]],
+            [p0[2],p1[2],p2[2],p3[2],p0[2]])
+        if c: self.view.text(c[0],c[1],c[2],s=s)
     def show(self):
         self.loop.create_task(self.update())
         self.loop.run_forever()
